# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2005 Dan Williams <dcbw@redhat.com> and Red Hat, Inc.


import string
import urllib
import os
import sys
import SSLCommon
from OpenSSL import SSL


class HTTPSURLopener(urllib.URLopener):
    def __init__(self, certs, timeout=None):
        self.ctx = SSLCommon.CreateSSLContext(certs)
        self._timeout = timeout
        urllib.URLopener.__init__(self)

    def open_https(self, url, data=None):
        """
        Need to override this because there was no visible
        way to get the SSL Context into open_https()
        """
        # Attempt to isolate host and request parts from URI
        host = None
        if type(url) is type(""):
            host, selector = urllib.splithost(url)
            if host:
                user_passwd, host = urllib.splituser(host)
                host = urllib.unquote(host)
        if not host:
            raise IOError, ('http error', 'no host given')

        h = SSLCommon.PlgHTTPS(host=host, ssl_context=self.ctx,
                timeout=self._timeout)

        h.putrequest('GET', selector)
        for args in self.addheaders:
            apply(h.putheader, args)

        try:
            h.endheaders()
        except SSL.Error, e:
            # Convert the SSL error into an IOError so the call fails
            raise IOError, e

        errcode, errmsg, headers = h.getreply()
        fp = h.getfile()
        if errcode == 200:
            return urllib.addinfourl(fp, headers, "https:" + url)
        else:
            return self.http_error(url, fp, errcode, errmsg, headers)

