#!/usr/bin/python -t
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2005 Dan Williams and Red Hat, Inc.
#

import SimpleHTTPServer
import threading
import urllib
import posixpath
import os
import sys
import SSLCommon
import time
import socket
from OpenSSL import SSL
import DebugUtils


class PlgHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):

    # For some reason, httplib closes the connection right after headers
    # have been sent if the connection is _not_ HTTP/1.1, which results in
    # a "Bad file descriptor" error when the client tries to read from the socket
    protocol_version = "HTTP/1.1"

    def __init__(self, request, client_address, server):
        self._server = server
        try:
            SimpleHTTPServer.SimpleHTTPRequestHandler.__init__(self, request, client_address, server)
        except socket.timeout:
            pass

    def list_directory(self, path):
        self.send_error(404, "No permission to list directory")

    def log_request(self, code='-', size='-'):
        # Don't log requests
        pass

    def translate_path(self, path):
        """Translate a /-separated PATH to the local filename syntax.

        Components that mean special things to the local file system
        (e.g. drive or directory names) are ignored.  (XXX They should
        probably be diagnosed.)

        This code is lifted from SimpleHTTPRequestHandler so that we can
        make sure the request is always based in our download directory,
        not the current directory.
        """
        path = posixpath.normpath(urllib.unquote(path))
        words = path.split('/')
        words = filter(None, words)
        path = self._server.http_dir
        for word in words:
            drive, word = os.path.splitdrive(word)
            head, word = os.path.split(word)
            if word in (os.curdir, os.pardir): continue
            path = os.path.join(path, word)
        return path

    if sys.version_info[:3] >= (2, 3, 0):
        def handle(self):
            """ Always close the connection when done """
            self.close_connection = 1

            try:
                self.handle_one_request()
            except SSL.Error, e:
                print e[0]
            except socket.timeout:
                pass


class PlgHTTPSServer(SSLCommon.PlgBaseSSLServer):
    def __init__(self, server_address, http_dir, certs):
        self.http_dir = http_dir
        SSLCommon.PlgBaseSSLServer.__init__(self, server_address, PlgHTTPRequestHandler, certs)

class PlgHTTPServer(SSLCommon.PlgBaseServer):
    def __init__(self, server_address, http_dir):
        self.http_dir = http_dir
        SSLCommon.PlgBaseServer.__init__(self, server_address, PlgHTTPRequestHandler)


class PlgHTTPServerManager(threading.Thread):
    def __init__(self, addr, http_dir, certs):
        if certs and len(certs) > 0:
            self._server = PlgHTTPSServer(addr, http_dir, certs)
        else:
            self._server = PlgHTTPServer(addr, http_dir)
        self._stopped = False
        threading.Thread.__init__(self)
        self.setName("PlgHTTPServerManager: %s/%s" % (addr, http_dir))

    def run(self):
        DebugUtils.registerThreadName(self)
        self._server.serve_forever()
        self._stopped = True

    def stop(self):
        self._server.stop()
        t = time.time()
        while not self._stopped:
            if time.time() > t + 2:
                break


###########################################################
# Testing stuff
###########################################################

if __name__ == '__main__':
    if len(sys.argv) < 4:
        print "Usage: python HTTPServer.py key_and_cert ca_cert peer_ca_cert"
        sys.exit(1)

    certs = {}
    certs['key_and_cert'] = sys.argv[1]
    certs['ca_cert'] = sys.argv[2]
    certs['peer_ca_cert'] = sys.argv[3]

    print "Starting..."

    srcdir = os.path.join("/tmp", "server_dir")
    if not os.path.exists(srcdir):
        os.makedirs(srcdir)

    print "Creating random test data..."
    f = open(os.path.join(srcdir, "testfile.dat"), "w")
    x = 1
    import random
    while x < 10000:
        try:
            f.write(os.urandom(50))
        except AttributeError:
            s = str(random.randint(0, 255))
            f.write(s)
        x = x + 1
    f.close()

    print "Starting the server."
    server = PlgHTTPServerManager(('localhost', 8886), srcdir, certs)
    server.start()

    while True:
        try:
            time.sleep(1)
        except KeyboardInterrupt:
            print "Quitting..."
            os._exit(0)
